/*
 * LEDD Project
 * Copyright (C) 2015 LEDD Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.idlegandalf.ledd;

import android.app.Application;
import android.content.Intent;
import android.preference.PreferenceManager;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.idlegandalf.ledd.callbacks.DiscoverCallback;
import com.idlegandalf.ledd.components.LedDDaemon;
import com.idlegandalf.ledd.helper.LedDHelper;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import timber.log.Timber;

public class ColorApplication extends Application {
    public static final String TAG = "LedD";
    public static final String INTENT_ACTION_REFRESH = "com.idlegandalf.ledd.action.refreshStripes";
    private static ColorApplication singleton;
    private HashMap<LedDDaemon, LedDHelper> ledDHelpers;

    public static ColorApplication getInstance() {
        return singleton;
    }

    @Override
    public void onCreate() {
        super.onCreate();

        singleton = this;
        ledDHelpers = new HashMap<>();
        this.onResume();
    }

    public LedDHelper getHelperForDaemon(final LedDDaemon ledDDaemon) {
        if (ledDHelpers.containsKey(ledDDaemon) && ledDHelpers.get(ledDDaemon) != null) {
            return ledDHelpers.get(ledDDaemon);
        } else {
            final LedDHelper dHelper = new LedDHelper(ledDDaemon, getApplicationContext());
            dHelper.discover(new DiscoverCallback() {
                @Override
                public void onConnectionFailed(String message) {

                }

                @Override
                public void onDiscoverSuccessfully(String version) {
                    ledDHelpers.put(ledDDaemon, dHelper);
                }
            });
            return dHelper;
        }
    }

    public List<LedDDaemon> getDaemons() {
        return new ArrayList<>(ledDHelpers.keySet());
    }

    public void teardown() {
        ledDHelpers.values().forEach(LedDHelper::teardown);

        PreferenceManager.getDefaultSharedPreferences(this).edit().putString("daemons", new Gson().toJson(ledDHelpers.keySet())).apply();
    }

    public void onResume() {
        if (PreferenceManager.getDefaultSharedPreferences(this).contains("daemons")) {
            Set<LedDDaemon> ledDDaemons = new Gson().fromJson(PreferenceManager.getDefaultSharedPreferences(this).getString("daemons", ""), new
                    TypeToken<Set<LedDDaemon>>() {
            }.getType());

            Timber.i("Loaded " + ledDDaemons.size() + " Daemons from preferences");

            for (final LedDDaemon ledDDaemon : ledDDaemons) {
                final LedDHelper helper = new LedDHelper(ledDDaemon, getApplicationContext());

                helper.discover(new DiscoverCallback() {
                    @Override
                    public void onDiscoverSuccessfully(String version) {
                        //ledDDaemon.setActive(true);
                        ledDHelpers.put(ledDDaemon, helper);

                        Intent i = new Intent(INTENT_ACTION_REFRESH);
                        sendBroadcast(i);
                    }

                    @Override
                    public void onConnectionFailed(String message) {
                        //ledDDaemon.setActive(false);
                        ledDHelpers.put(ledDDaemon, helper);

                        Intent i = new Intent(INTENT_ACTION_REFRESH);
                        sendBroadcast(i);
                    }
                });
            }
        }
    }
}
