/*
 * LEDD Project
 * Copyright (C) 2015 LEDD Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.idlegandalf.ledd.components;

import com.thetransactioncompany.jsonrpc2.JSONRPC2Request;
import com.thetransactioncompany.jsonrpc2.JSONRPC2Response;

import org.json.JSONException;

import lombok.Getter;

@Getter
public class Sendable {
    LedDDaemon recipient;
    JSONRPC2Request request;
    AnswerTask onAnswer;

    public Sendable(JSONRPC2Request request, AnswerTask task, LedDDaemon recipient) throws JSONException {
        this.request = request;
        this.onAnswer = task;
        this.recipient = recipient;
    }

    public void onResponse(JSONRPC2Response object) {
        if (onAnswer != null) {
            onAnswer.onResponse(object);
        }
    }

    public void onNoResponse() {
        onAnswer.onConnectionFailed("no_response");
    }

    public void onConnectionFailed(String message) {
        if (onAnswer != null) {
            onAnswer.onConnectionFailed(message);
        }
    }
}
