package com.idlegandalf.ledd.fragments;

/*
 * LEDD Project
 * Copyright (C) 2015 LEDD Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import android.app.Dialog;
import android.app.DialogFragment;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.google.android.material.textfield.TextInputLayout;
import com.idlegandalf.ledd.ColorActivity;
import com.idlegandalf.ledd.R;
import com.idlegandalf.ledd.components.LedStripe;
import com.idlegandalf.ledd.components.Profile;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;


public class AddProfileDialog extends DialogFragment implements DialogInterface.OnShowListener {

    @BindView(R.id.input_profile_name_layout)
    TextInputLayout nameLayout;
    @BindView(R.id.input_profile_name)
    EditText nameText;
    @BindView(R.id.container_linlay)
    LinearLayout checkboxContainer;
    List<LedStripe> mStripes;
    List<CheckBox> mCheckboxes;

    public Dialog onCreateDialog(Bundle savedInstanceState) {
        View v = View.inflate(getActivity(), R.layout.fragment_addprofile, null);
        ButterKnife.bind(this, v);

        mStripes = ((ColorActivity) getActivity()).getStripes();
        mCheckboxes = new ArrayList<>();

        if (mStripes != null) {
            for (LedStripe ledStripe : mStripes) {
                CheckBox nC = (CheckBox) View.inflate(getActivity(), R.layout.fragment_addprofile_item, null);

                nC.setText(ledStripe.getName());
                mCheckboxes.add(nC);
                checkboxContainer.addView(nC);
            }
        }

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());
        builder.setView(v).setPositiveButton("Save", null).setNegativeButton("Cancel", null);

        AlertDialog dialog = builder.create();

        dialog.setOnShowListener(this);

        return dialog;
    }

    @Override
    public void onShow(final DialogInterface dialog) {

        final AlertDialog alertDialog = (AlertDialog) dialog;

        nameText.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                nameLayout.setError(null);
            }
        });

        alertDialog.getButton(AlertDialog.BUTTON_NEGATIVE).setOnClickListener(v -> alertDialog.dismiss());

        alertDialog.getButton(AlertDialog.BUTTON_POSITIVE).setOnClickListener(v -> {
            if (nameText.getText().toString().isEmpty()) {
                nameLayout.setError("Please enter a name");
                return;
            }

            int checked = 0;
            for (CheckBox c : mCheckboxes) {
                if (c.isChecked()) {
                    checked++;
                    mStripes.get(checkboxContainer.indexOfChild(c)).getColor();
                }
            }

            if (checked < 2) {
                Toast.makeText(getActivity(), "Please select at least two stripes for your profile", Toast.LENGTH_LONG).show();
                return;
            }

            Profile profile = new Profile(nameText.getText().toString());

            mCheckboxes.stream().filter(CompoundButton::isChecked).forEach(c -> {
                LedStripe stripe = mStripes.get(checkboxContainer.indexOfChild(c));
                profile.getValues().append(stripe.getId(), stripe.getColor());
            });

            ((ColorActivity) getActivity()).addProfile(profile);
            Toast.makeText(getActivity(), "Added profile " + nameText.getText().toString(), Toast.LENGTH_SHORT).show();
            dismiss();
        });
    }
}

